﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using System.Web.Mvc;
using TengoPlan4U.Models;

namespace TengoPlan4U.Controllers
{
    public class PlansController : Controller
    {
        private TengoPlanEntities db = new TengoPlanEntities();

        // GET: Plans
        public ActionResult TuPlan(string direccion, string tipoPlan)
        {
            //var planes = db.Planes.Include(p => p.Ciudad).Include(p => p.TipoPlan);
            string coordinates = Comunes.GetCoordinates(direccion);
            List<Local> listaLocales = Comunes.GetPlaces(coordinates, tipoPlan);
            return View(listaLocales.OrderByDescending(l => l.Rating));
        }

        public ActionResult IrOpiniones(string idLocal)
        {
            return RedirectToAction("VerOpiniones", "Opinion", new { placeId = idLocal });
        }


        // GET: Plans/Details/5
        public ActionResult Details(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Plan plan = db.Planes.Find(id);
            if (plan == null)
            {
                return HttpNotFound();
            }
            return View(plan);
        }

        // GET: Plans/Create
        public ActionResult Create()
        {
            ViewBag.CiudadId = new SelectList(db.Ciudades, "CiudadId", "Nombre");
            ViewBag.TipoPlanId = new SelectList(db.TiposPlanes, "TipoPlanId", "Nombre");
            return View();
        }

        // POST: Plans/Create
        // Para protegerse de ataques de publicación excesiva, habilite las propiedades específicas a las que desea enlazarse. Para obtener 
        // más información vea http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create([Bind(Include = "PlanId,TipoPlanId,CiudadId,Zona")] Plan plan)
        {
            if (ModelState.IsValid)
            {
                db.Planes.Add(plan);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            ViewBag.CiudadId = new SelectList(db.Ciudades, "CiudadId", "Nombre", plan.CiudadId);
            ViewBag.TipoPlanId = new SelectList(db.TiposPlanes, "TipoPlanId", "Nombre", plan.TipoPlanId);
            return View(plan);
        }

        // GET: Plans/Edit/5
        public ActionResult Edit(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Plan plan = db.Planes.Find(id);
            if (plan == null)
            {
                return HttpNotFound();
            }
            ViewBag.CiudadId = new SelectList(db.Ciudades, "CiudadId", "Nombre", plan.CiudadId);
            ViewBag.TipoPlanId = new SelectList(db.TiposPlanes, "TipoPlanId", "Nombre", plan.TipoPlanId);
            return View(plan);
        }

        // POST: Plans/Edit/5
        // Para protegerse de ataques de publicación excesiva, habilite las propiedades específicas a las que desea enlazarse. Para obtener 
        // más información vea http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit([Bind(Include = "PlanId,TipoPlanId,CiudadId,Zona")] Plan plan)
        {
            if (ModelState.IsValid)
            {
                db.Entry(plan).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.CiudadId = new SelectList(db.Ciudades, "CiudadId", "Nombre", plan.CiudadId);
            ViewBag.TipoPlanId = new SelectList(db.TiposPlanes, "TipoPlanId", "Nombre", plan.TipoPlanId);
            return View(plan);
        }

        // GET: Plans/Delete/5
        public ActionResult Delete(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Plan plan = db.Planes.Find(id);
            if (plan == null)
            {
                return HttpNotFound();
            }
            return View(plan);
        }

        // POST: Plans/Delete/5
        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            Plan plan = db.Planes.Find(id);
            db.Planes.Remove(plan);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                db.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}
