﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using TengoPlan4U.Models;

namespace TengoPlan4U.Models
{
    public static class Comunes
    {
        private static TengoPlanEntities _tengoPlanDb = new TengoPlanEntities();

        /// <summary>
        /// Obtiene las coordenadas de una determinada dirección o zona
        /// </summary>
        /// <param name="direccion"></param>
        /// <returns>string con latitud y longitud</returns>
        public static string GetCoordinates(string direccion)
        {
            string API_KEY = string.Empty;
            if(DateTime.Now.Second % 2 == 0)
            {
                API_KEY = Estructuras.API_KEY;
            }
            else API_KEY = Estructuras.API_KEY_2;
            string googleGeo = string.Format(Estructuras.ServiceGeocode, direccion, API_KEY);
            var result = new System.Net.WebClient().DownloadString(googleGeo);
            GoogleGeocode.Rootobject googleData = JsonConvert.DeserializeObject<GoogleGeocode.Rootobject>(result);
            float latitud = googleData.results[0].geometry.location.lat;
            float longitud = googleData.results[0].geometry.location.lng;
            return latitud.ToString().Replace(",", ".") + ", " + longitud.ToString().Replace(",", ".");
        }

        /// <summary>
        /// Obtiene la lista de locales situados en las coordenadas pasadas como parámetros que ofrecen el plan seleccionado
        /// </summary>
        /// <param name="coordinates"></param>
        /// <param name="tipoPlan"></param>
        /// <returns>lista de locales</returns>
        public static List<Local> GetPlaces(string coordinates, string tipoPlan)
        {
            string API_KEY = string.Empty;
            if (DateTime.Now.Second % 2 == 0)
            {
                API_KEY = Estructuras.API_KEY;
            }
            else API_KEY = Estructuras.API_KEY_2;
            string types = GetTypesPlan(tipoPlan);
            List<Local> listaLocales = new List<Local>();
            string googlePlaces = string.Format(Estructuras.ServicePlacesRadar, coordinates, types, API_KEY);
            WebClient wc = new WebClient();
            wc.Encoding = Encoding.UTF8;
            var result = wc.DownloadString(googlePlaces);
            GooglePlacesRadar.Rootobject googleData = JsonConvert.DeserializeObject<GooglePlacesRadar.Rootobject>(result);
            foreach (var place in googleData.results)
            {
                GooglePlaceDetails.Rootobject placeDetails = GetPlaceDetails(place.place_id);
                if (placeDetails.status.Equals("OK"))
                {
                    Local local = new Local();
                    local.Nombre = placeDetails.result.name;
                    local.GooglePlaceId = placeDetails.result.place_id;
                    local.Direccion = placeDetails.result.formatted_address;
                    local.Rating = placeDetails.result.rating;
                    local.UrlGoogleMaps = placeDetails.result.url;
                    if (!string.IsNullOrEmpty(placeDetails.result.website))
                    {
                        local.Website = placeDetails.result.website;
                    }
                    else
                    {
                        local.Website = "sin-web";
                    }
                    if (placeDetails.result.photos != null)
                    {
                        local.UrlImagen = string.Format(Estructuras.ServicesPhotos, 300, placeDetails.result.photos[0].photo_reference, API_KEY);
                    }
                    else
                    {
                        local.UrlImagen = "../Images/tengoPlan.png";
                    }
                    if (local.Rating > 0) listaLocales.Add(local);
                }
            }
            return listaLocales;
        }

        public static string GetTypesPlan(string tipoPlan)
        {
            string types = string.Empty;
            switch (tipoPlan)
            {
                case "Caña + tapas":
                    types = "bar";
                    break;
                case "Desayunos":
                    types = "bar|cafe";
                    break;
                case "Menú del día":
                    types = "restaurant|food";
                    break;
            }
            return types;
        }

        /// <summary>
        /// Obtiene lista de locales según sus ids pasados como parámetro
        /// </summary>
        /// <param name="localesIds"></param>
        /// <returns>Lista de locales recomendados</returns>
        public static List<Local> GetRecomendaciones(List<string> localesIds)
        {
            string API_KEY = string.Empty;
            if (DateTime.Now.Second % 2 == 0)
            {
                API_KEY = Estructuras.API_KEY;
            }
            else API_KEY = Estructuras.API_KEY_2;
            List<Local> listaLocales = new List<Local>();
            foreach (var id in localesIds)
            {
                GooglePlaceDetails.Rootobject placeDetails = GetPlaceDetails(id);
                if (placeDetails.status.Equals("OK"))
                {
                    Local local = new Local();
                    local.Nombre = placeDetails.result.name;
                    local.GooglePlaceId = placeDetails.result.place_id;
                    local.Direccion = placeDetails.result.formatted_address;
                    local.Rating = placeDetails.result.rating;
                    local.UrlGoogleMaps = placeDetails.result.url;
                    if (!string.IsNullOrEmpty(placeDetails.result.website))
                    {
                        local.Website = placeDetails.result.website;
                    }
                    else
                    {
                        local.Website = "sin-web";
                    }
                    if (placeDetails.result.photos != null)
                    {
                        local.UrlImagen = string.Format(Estructuras.ServicesPhotos, 300, placeDetails.result.photos[0].photo_reference, API_KEY);
                    }
                    else
                    {
                        local.UrlImagen = "../Images/tengoPlan.png";
                    }
                    if (local.Rating > 0) listaLocales.Add(local);
                }
            }
            return listaLocales;
        }

        /// <summary>
        /// Obtiene detalles de un local con el id pasado como parámetro
        /// </summary>
        /// <param name="placeId"></param>
        /// <returns></returns>
        public static GooglePlaceDetails.Rootobject GetPlaceDetails(string placeId)
        {
            string API_KEY = string.Empty;
            if (DateTime.Now.Second % 2 == 0)
            {
                API_KEY = Estructuras.API_KEY;
            }
            else API_KEY = Estructuras.API_KEY_2;
            string googlePlaceDetails = string.Format(Estructuras.ServicePlaceDetails, placeId, API_KEY);
            WebClient wc = new WebClient();
            wc.Encoding = Encoding.UTF8;
            var result = wc.DownloadString(googlePlaceDetails);
            GooglePlaceDetails.Rootobject googleData = JsonConvert.DeserializeObject<GooglePlaceDetails.Rootobject>(result);
            return googleData;
        }

        public static Recomendacion GetRecomendaciones(string email)
        {
            UsuarioRegistrado user = _tengoPlanDb.UsuariosRegistrados.Where(u => u.Email.Equals(email)).FirstOrDefault();
            Dictionary<string, int> valoresUsuario = new Dictionary<string, int>();
            Dictionary<int, double> correlaciones = new Dictionary<int, double>();
            List<Opinion> opUsuario = _tengoPlanDb.Opiniones.Where(o => o.UsuarioId.Equals(user.UsuarioRegistradoId)).ToList();

            if (opUsuario.Count > 0)
            {
                List<int> usuariosIds = _tengoPlanDb.Opiniones.Where(p => p.UsuarioId != user.UsuarioRegistradoId)
                    .Select(o => o.UsuarioId).Distinct().ToList();
                if (usuariosIds.Count > 0)
                {
                    foreach (int id in usuariosIds)
                    {
                        List<Opinion> opAux = _tengoPlanDb.Opiniones.Where(o => o.UsuarioId.Equals(id)).ToList();
                        Dictionary<string, int> valoresOtro = new Dictionary<string, int>();
                        valoresUsuario.Clear();
                        foreach (Opinion op in opUsuario)
                        {
                            foreach (Opinion op2 in opAux)
                            {
                                if (op.GooglePlaceId.Equals(op2.GooglePlaceId))
                                {
                                    valoresUsuario[op.GooglePlaceId] = op.Rating;
                                    valoresOtro[op2.GooglePlaceId] = op2.Rating;
                                }
                            }
                        }
                        if (valoresUsuario.Count > 0)
                        {
                            correlaciones[id] = CoeficientePearson(valoresUsuario, valoresOtro);
                            int idForMaxCorrelation = correlaciones.FirstOrDefault(x => x.Value == correlaciones.Values.Max()).Key;
                            if (correlaciones[idForMaxCorrelation] >= 0)
                            {
                                List<Opinion> opUsuarioAfin = _tengoPlanDb.Opiniones.Where(o => o.UsuarioId.Equals(idForMaxCorrelation)).ToList();
                                List<string> localesUsuario = opUsuario.Select(o => o.GooglePlaceId).Distinct().ToList();
                                List<string> localesUserAfin = opUsuarioAfin.Select(o => o.GooglePlaceId).Distinct().ToList();
                                List<string> places = localesUserAfin.Except(localesUsuario).ToList();
                                //chequear correlaciones, el mayor valor, coger id y obtener opioniones de ese usuario, coger los locales
                                //que no tenga usuario actual y devolverlos recomendados
                                Recomendacion result = new Recomendacion
                                {
                                    UsuarioId = user.UsuarioRegistradoId,
                                    Locales = places
                                };
                                return result;
                            }
                            else return null;
                        }
                        else return null;
                    }
                    
                }
                return null;
            }
            return null;
        }

        /// <summary>
        /// Calcula la correlación de pearson para establecer la similitud entre dos vectores de valores
        /// </summary>
        /// <returns></returns>
        public static double CoeficientePearson(Dictionary<string, int> valores1, Dictionary<string, int> valores2)
        {
            int[] vector1 = valores1.Values.ToArray();
            int[] vector2 = valores2.Values.ToArray();

            double[] vector_xy = new double[vector1.Length];
            double[] vector_xp2 = new double[vector1.Length];
            double[] vector_yp2 = new double[vector1.Length];
            for (int i = 0; i < vector1.Length; i++)
                vector_xy[i] = vector1[i] * vector2[i];
            for (int i = 0; i < vector1.Length; i++)
                vector_xp2[i] = Math.Pow(vector1[i], 2.0);
            for (int i = 0; i < vector1.Length; i++)
                vector_yp2[i] = Math.Pow(vector2[i], 2.0);
            double sum_x = 0;
            double sum_y = 0;
            foreach (double n in vector1)
                sum_x += n;
            foreach (double n in vector2)
                sum_y += n;
            double sum_xy = 0;
            foreach (double n in vector_xy)
                sum_xy += n;
            double sum_xpow2 = 0;
            foreach (double n in vector_xp2)
                sum_xpow2 += n;
            double sum_ypow2 = 0;
            foreach (double n in vector_yp2)
                sum_ypow2 += n;
            double Ex2 = Math.Pow(sum_x, 2.00);
            double Ey2 = Math.Pow(sum_y, 2.00);

            double den = Math.Sqrt((vector1.Length * sum_xpow2 - Ex2) * (vector1.Length * sum_ypow2 - Ey2));
            double num = (vector1.Length * sum_xy - sum_x * sum_y);
            if(den == 0)
            {
                return 0;
            }
            double coeficiente = num /den;

            return coeficiente;
        }
    }
}